<?php

namespace HCM\FPA\Calculos\Folha;

use HCM\FPA\Entity\FpaitecalcfolhaEntity;
use HCM\FPA\Entity\FpatipomovimentEntity;
use HCM\Util\MsgCalculo;
use HCM\Util\Repositories;

class VerificacaoPessoa
{
    private $calculoFolha;

    public function __construct(CalculoFolha $calculoFolha)
    {
        $this->calculoFolha = $calculoFolha;
    }

    public function pessoaJaPossuiLiquido(): bool
    {
        $vinculosDaMesmaPessoa = $this->getVinculosDaMesmaPessoa();
        if (!empty($vinculosDaMesmaPessoa)) {
            foreach ($vinculosDaMesmaPessoa as $vinculoMesmaPessoa) {
                if ($this->vinculoTemLiquido($vinculoMesmaPessoa["NRVINCULOM"])) {
                    $msg = 'Já existe líquido para esta mesma pessoa para o vínculo ' . $vinculoMesmaPessoa["NRVINCULOM"] . '. Favor mudar a ocorrência.';

                    MsgCalculo::getInstance()->adicionaMensagem($msg, MsgCalculo::ERRO);
                    return true;
                }
            }
        }

        return false;
    }

    private function getVinculosDaMesmaPessoa(): array
    {
        $sql = $this->calculoFolha->entityManager->getConnection()->prepare("
            SELECT *
            FROM GPE_VINCULOH VH
            WHERE NRVINCULOM IN (SELECT NRVINCULOM FROM GPE_VINCULOM WHERE NRPESSOA = :P_NRPESSOA AND NRORG = :P_NRORG)
              AND NRVINCULOM <> :P_NRVINCULOM
              AND VH.NRORG = :P_NRORG
              AND VH.DTMESCOMPETENC = (SELECT MAX(H1.DTMESCOMPETENC)
                                       FROM GPE_VINCULOH H1
                                       WHERE H1.NRORG = :P_NRORG
                                         AND H1.NRVINCULOM = VH.NRVINCULOM
                                         AND H1.DTMESCOMPETENC = :P_DTMESCOMPETENC)
        ");

        $sql->execute(
            array(
                'P_NRORG' => $this->calculoFolha->_nrorg,
                'P_NRVINCULOM' => $this->calculoFolha->_vinculo->getNrvinculom(),
                'P_NRPESSOA' => $this->calculoFolha->_vinculo->getNrpessoa(),
                'P_DTMESCOMPETENC' => $this->calculoFolha->_mes_competencia->format('d/m/Y')
            )
        );
        return $sql->fetchAll();
    }

    private function vinculoTemLiquido($NRVINCULOM): bool
    {
        $calc_folhaAtual = $this->calculoFolha->entityManager->getRepository(Repositories::FPA_CALCULOFOLHA)->retornaCalculoFolha(
            $this->calculoFolha->_nrorg, FpatipomovimentEntity::TP_MOV_MENSAL, 1, $this->calculoFolha->_nrOcorrenciaCalculo, $this->calculoFolha->_mes_competencia, $this->calculoFolha->_mes_competencia
        )[0];

        $itemCalcCompetenciaAtual = new FpaitecalcfolhaEntity($this->calculoFolha->entityManager, $this->calculoFolha->hcmParameters, $calc_folhaAtual->getNrcalculofolha());
        $itemCalcCompetenciaAtual->retornaItensPorVinculo($this->calculoFolha->_vinculo->getNrorg(), $NRVINCULOM, $calc_folhaAtual->getNrcalculofolha());

        return $itemCalcCompetenciaAtual->verificaLiquido(
            $this->calculoFolha->getAssocEventos(), $this->calculoFolha->_eventom, $this->calculoFolha->_nrorg
        );
    }
}