<?php

namespace HCM\FPA\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;
use HCM\Util\Util;

class TabelaFolhaController extends Crud {

    protected $tabelaFolhaService;
    protected $dataSourceName = 'fpa_tabelafolha_get';
    protected $dataSourceName1 = 'fpa_faixatabela_get';
    
    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO', 'DTINIVIGENCIA', 'P_NRTABELAFOLHA', 'P_DTMESCOMPETENC','IDATIVO');
    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, \HCM\FPA\Services\TabelaFolhaService $tabelaFolhaService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->tabelaFolhaService = $tabelaFolhaService;
        parent::__construct($dataSourceManager);
    }

    public function getTabelaFolha(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function getFaixaTabela(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName1);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function setTabelaFolha(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $rows = $request->getDataSet()->getRows();
            foreach ($rows as &$row) {
                if (array_key_exists('NRTABELAFOLHA', $row)) {
                    if ($row['NRTABELAFOLHA'] === '') {
                        $row['NRTABELAFOLHA'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'FPA_TABELAFOLHA', 1, 12, true);
                    }
                } else {
                        $row['NRTABELAFOLHA'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'FPA_TABELAFOLHA', 1, 12, true);
                }
            }
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function setFaixaTabela(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $this->dataSourceName = "fpa_faixatabela";
            $rows = $request->getDataSet()->getRows();
            // Se houver o registro 'Reajuste', significa que esse post é para atualização das
            // faixas de um registro, e não a adição ou edição de uma faixa
            if(isset($rows[0]['Reajuste'])){
                $this->tabelaFolhaService->reajustaValores($rows[0]);        
            }else{
                foreach ($rows as &$row) {
                    if (array_key_exists('NRFAIXATABFOLHA', $row)) {
                        if ($row['NRFAIXATABFOLHA'] === '') {
                        $row['NRFAIXATABFOLHA'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'FPA_FAIXATABELA', 1, 12, true);
                        }
                    } else {
                        $row['NRFAIXATABFOLHA'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'FPA_FAIXATABELA', 1, 12, true);
                    }
                }
                parent::save($request, $response);
            }    
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Função cujo propósito é realizar a duplicação de um registro da tabela folha juntamente
     * com suas faixas
     */
    public function setReplicarFolhaFaixas(DTO\Request\DataSet $request, DTO\Response $response){
        $rows = $request->getDataSet()->getRows();
        $rows[0]['newNRTABELAFOLHA'] = \Geral\Util\NovoCodigo::selectMax('FPA_TABELAFOLHA','NRTABELAFOLHA');
        $this->tabelaFolhaService->replicarFolhaFaixas($rows[0]);
    }        
    
    public function deleteTabelaFolha(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $dataset = $request->getDataSet()->getRows();
            $dataset = $dataset[0];
            $this->tabelaFolhaService->deleteTabelaFolha($dataset);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('delete', array(true)));
        } catch(\Exception $e) {
            if ($e->getCode() === 10) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            }  
            else {
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
 	        }
        }
    }
    
    public function deleteFaixaTabela(DTO\Request\DataSet $request, DTO\Response $response){
        $this->dataSourceName = "fpa_faixatabela";
        parent::delete($request, $response);
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $condition = $this->validaCondition($condition);
            
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }

    private function validaCondition($condition){
        if($condition['columnName'] == 'NRTABELAFOLHA' && $condition['value'] == '%4%' && $condition['operator'] == 'LIKE_I'){
            $condition['value'] = '4' ;
            $condition['operator'] = '>';
        }
        if($condition['columnName'] == 'NRTABELAFOLHA' && $condition['value'] == '%2,5%' && $condition['operator'] == 'LIKE_I'){
            $condition['value'] = [2, 5] ;
            $condition['operator'] = 'IN';
        }
        
        return $condition;
    }    
} 