<?php

namespace HCM\FPA\Services;

use Doctrine\ORM\EntityManager;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;
use HCM\Util\Repositories;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO;
use HCM\Util\DateUtil;
use HCM\FPA\Repository\GozoFeriasRepository;
use HCM\Geral\Repository\AlteSituacaoFuncionalRepository;
use HCM\Geral\Services\ESocialService;
use HCM\Geral\Services\VinculoServices;
use Teknisa\Model\Gestaopessoas\Entities\GpeSitufuncm;
use HCM\FPA\Services\ExportacaoExcelService;
use HCM\Util\FPA;
use Zeedhi\Framework\HTTP\Kernel;
use Folha\Entities\GpeVinculom;
use HCM\FPA\Entity\FpagozoferiasEntity;
use HCM\FPA\Entity\FpaferiasEntity;

class GozoFeriasService {
    
    protected $entityManager;
    protected $hcmParameters;
    protected $gozoFeriasRepository;
    protected $alteSituacaoFuncionalRepository;
    protected $fpaGozoFeriasEntity;
    protected $fpaferiasEntity;
    
    public function __construct(EntityManager $entityManager, HCMParameters $hcmParameters, GozoFeriasRepository $gozoFeriasRepository, AlteSituacaoFuncionalRepository $alteSituacaoFuncionalRepository, ESocialService $eSocialService,
    ExportacaoExcelService $exportacaoExcelService, Kernel $httpKernel, VinculoServices $vinculoServices) {
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
        $this->gozoFeriasRepository = $gozoFeriasRepository;
        $this->eSocialService = $eSocialService;
        $this->alteSituacaoFuncionalRepository = $alteSituacaoFuncionalRepository;
        $this->exportacaoExcelService = $exportacaoExcelService;
        $this->httpKernel = $httpKernel;
        $this->vinculoServices = $vinculoServices;
        
        $this->fpaGozoFeriasEntity = new FpagozoferiasEntity($entityManager, $hcmParameters);
        $this->fpaferiasEntity = new FpaferiasEntity($entityManager, $hcmParameters);
    }
    
    public function deleteGozoFerias($dataset) {
        $nrFerias = $dataset["NRFERIAS"];
        $gozoFerias = $this->entityManager->getRepository(Repositories::FPA_GOZOFERIAS)->findBy(array("nrferias" => $nrFerias, "nrorg" => $dataset['NRORG']));
        if (isset($gozoFerias[0])) {
            throw new \Exception(Messages::HCM01000_FPA_FERIAS_EXISTS_GOZOFERIAS_ERROR, 10);
        }
        else {
            $this->entityManager->beginTransaction();
            $ferias = $this->entityManager->getRepository(Repositories::FPA_FERIAS)->findOneBy(array("nrferias" => $nrFerias, "nrorg" => $dataset['NRORG']));
            $this->entityManager->remove($ferias);
            $this->entityManager->flush();
            $this->entityManager->commit();
        }
    }
    
    public function verificaPossibilidadeAlteracao($nrchave, $edicaoexclusao) {
        $edicaodisponivel = $this->eSocialService->verificaPossibilidadeAlteracao($nrchave, 'Gozo de Ferias', $edicaoexclusao);
         
        if ($edicaodisponivel === true){
            return true;
        } else {
            $message = Messages::evaluateMessage(MESSAGES::HCM00000_EDITAR_EXCLUIR_REGISTRO_AFASTAMENTO, $nrchave);
            return new \Zeedhi\Framework\DTO\Response\Message($message, DTO\Response\Message::TYPE_WARNING);
        }
    }
    
    public function validaGozoFerias($dataset) {
        
        $nrVinculom = $dataset["NRVINCULOM"];
        $nrgozoferias = $dataset['NRGOZOFERIAS'];
        $inigozoatual = DateUtil::getDataDeString($dataset['DTINIGOZOFERIAS']); 
        $fimgozoatual = DateUtil::getDataDeString($dataset['DTFIMGOZOFERIAS']);
        $nrorg = $dataset['NRORG'];
        $gozoFerias = $this->gozoFeriasRepository->retornaGozoFerias($nrorg,$nrVinculom);
        foreach($gozoFerias as $periodo){
            if($periodo['DTINIGOZOFERIAS'] && $periodo['DTFIMGOZOFERIAS']){
                $inigozo =  DateUtil::getDataDeString($periodo['DTINIGOZOFERIAS'],DateUtil::FORMATO_BRASILEIRO_DATAHORA);
                $fimgozo =  DateUtil::getDataDeString($periodo['DTFIMGOZOFERIAS'],DateUtil::FORMATO_BRASILEIRO_DATAHORA);
                if ($nrgozoferias == $periodo['NRGOZOFERIAS']) {
                    continue;
                }
                $datas = array(DateUtil::truncateString($periodo['DTINIGOZOFERIAS']),DateUtil::truncateString($periodo['DTFIMGOZOFERIAS']));
                if ($inigozoatual) {
                    if (DateUtil::comparaDatas($inigozoatual,$inigozo,DateUtil::DEPOIS) && DateUtil::comparaDatas($inigozoatual,$fimgozo,DateUtil::ANTES) || 
                        DateUtil::comparaDatas($fimgozoatual,$inigozo,DateUtil::DEPOIS) && DateUtil::comparaDatas($fimgozoatual,$fimgozo,DateUtil::ANTES)) {
                        throw new \Exception(Messages::evaluateMessage(Messages::HCM01000_FPA_GOZOFERIAS_VINCPOSSUIPERIODO_ERROR,$datas),10);
                    }
                }
            }
        }
    }



    public function getVinculoAptoFerias($row, $nrorg) {
       
        $dataFimAquisicaoFerias = $this->gozoFeriasRepository->getVinculoAptoFerias($row, $nrorg)->getDtfimaquisicao();

        $dataInicioGozo =  DateUtil::getDataDeString($row['DTINIGOZOFERIAS'],DateUtil::FORMATO_BRASILEIRO, true);

        return $dataInicioGozo > $dataFimAquisicaoFerias;

 
    }
    
    public function avisaGozoAfastados($dataset) {
        
        $nrVinculom = $dataset["NRVINCULOM"];
        $nrgozoferias = $dataset['NRGOZOFERIAS'];
        $inigozoatual = DateUtil::getDataDeString($dataset['DTINIGOZOFERIAS']); 
        $fimgozoatual = DateUtil::getDataDeString($dataset['DTFIMGOZOFERIAS']);
        $nrorg = $dataset['NRORG'];
        $alteSituFunc = $this->alteSituacaoFuncionalRepository->retornaUltimaSituacaoPorDatainicio($nrorg, $nrVinculom, $fimgozoatual);
        if ($alteSituFunc) {
            // Verifica se não são as situações Normal e Atestado médico
            if (($alteSituFunc->getNrsitufuncm() != GpeSitufuncm::SITUACAO_FUNCIONAL_NORMAL) &&
                ($alteSituFunc->getNrsitufuncm() != GpeSitufuncm::SITUACAO_FUNCIONAL_ATESTADO)) {
                    
                // Verifica se o período de gozo e o período do afastamento se intercalam
                $dtinisitufunc = $alteSituFunc->getDtinisitufunc();
                if ($alteSituFunc->getDtfimsitufunc())
                    $dtfimsitufunc = $alteSituFunc->getDtfimsitufunc();
                else
                    $dtfimsitufunc = $fimgozoatual; 
                if (DateUtil::comparaDatas($inigozoatual,$dtinisitufunc,DateUtil::DEPOIS) && DateUtil::comparaDatas($inigozoatual,$dtfimsitufunc,DateUtil::ANTES) || 
                    DateUtil::comparaDatas($fimgozoatual,$dtinisitufunc,DateUtil::DEPOIS) && DateUtil::comparaDatas($fimgozoatual,$dtfimsitufunc,DateUtil::ANTES)){
                        $dataAlteSituFunc = array(DateUtil::truncateString($dtinisitufunc),DateUtil::truncateString($dtfimsitufunc));
                        return $dataAlteSituFunc;
                }
            }
        }
        return false;
    }
    
     /*Planilhas*/
    /*Ferias*/
    public function baixarPlanilhaFerias($nrorg, $nmarquivo) {
        $cabecalho = array(
            "Nr. Vínculo",
            "Matrícula",
            "Matrícula eSocial",
            "Início Aquisição",
            "Fim Aquisição",
            "Mês Afastamento",
            "Status férias",
            "Dias Afastamento",
            "Início gozo",
            "Fim gozo",
            "Dt Retorno Férias",
            "Dias Abono",
            "Qtd Faltas mês"
        );
        
        $dtmescompetenc = $this->hcmParameters->getDtCompetencia();
        
        /*$vinculos = $this->getListaVinculos($nrorg, $dtmescompetenc);
        if(count($vinculos) == 0) {
            $vinculos[0][] = "9999";
            $vinculos[0][] = "";
            $vinculos[0][] = "";
            $vinculos[0][] = "Nome do vínculo";
        }
        $vinculos[0][] = "31";
        $vinculos[0][] = "912345678";
        $vinculos[0][] = "31";
        $vinculos[0][] = "998765432";
        $vinculos[0][] = "nomedeemail@servidor.com";
        $vinculos[0][] = "31";
        $vinculos[0][] = "21211234";
        $vinculos[0][] = "";
        $vinculos[0][] = "";
        $vinculos[0][] = "31";
        $vinculos[0][] = "21215678";*/
        
        return $this->exportacaoExcelService->gravarTemplate_1($nrorg, $cabecalho, $nmarquivo);
    }
    
    public function importarPlanilhaFerias($row) {
        $nrorg       = $this->hcmParameters->getNrorgTrabalho();
        $nrorgPadrao = $this->hcmParameters->getNrorgPadrao();
        $messages    = array(); 
        $dados       = $this->getDadosFerias($row['DSPATH'][0]);

        foreach($dados as $lin => $dado) {
            $res = $this->insereFerias($lin + 2, $nrorg, $nrorgPadrao,
                $dado['NRVINCULOM'],
                $dado['CDMATRICULA'],
                $dado['CDMATRICULAESOCIAl'],
                $dado['DTINIAQUISICAO'],
                $dado['DTFIMAQUISICAO'],
                $dado['QTMESAFASTAMENT'],
                $dado['IDCONTROLFERIAS'],
                $dado['QTDIAAFASTAMENT'],
                $dado['DTINIGOZOFERIAS'],
                $dado['DTFIMGOZOFERIAS'],
                $dado['DTRETORNOFERIAS'],
                $dado['QTDIASABONOFERI'],
                $dado['QTFALTASPERIODO']);
            foreach($res as $msg)
                $messages[] = $msg;
        }
        return $messages;
    }
    
    public function getDadosFerias($path) {
        $head = array(
                'NRVINCULOM',
                'CDMATRICULA',
                'CDMATRICULAESOCIAl',
                'DTINIAQUISICAO',
                'DTFIMAQUISICAO',
                'QTMESAFASTAMENT',
                'IDCONTROLFERIAS',
                'QTDIAAFASTAMENT',
                'DTINIGOZOFERIAS',
                'DTFIMGOZOFERIAS',
                'DTRETORNOFERIAS',
                'QTDIASABONOFERI',
                'QTFALTASPERIODO'
            );
        
        $file = $this->httpKernel->factoryFromFileData($path, FPA::getTempDirectory()."/");
        $caminhoNovo = DIRECTORY_SEPARATOR . $file->getPathname();
        $dateColumns = array('DTINIAQUISICAO', 'DTFIMAQUISICAO', 'DTINIGOZOFERIAS', 'DTFIMGOZOFERIAS', 'DTRETORNOFERIAS');
        
        $file = \PHPExcel_IOFactory::load($caminhoNovo);
        $file->setActiveSheetIndex(0);

        return $this->exportacaoExcelService->preparaInfo_generic($file, $head, array('NRVINCULOM', 'CDMATRICULA', 'CDMATRICULAESOCIAl'), $dateColumns);
    }
    
    public function insereFerias($nrRow, $nrorg, $nrorgPadrao, $nrvinculom, $cdmatricula, $cdmatriculaesocial, $dtIniaquisicao,
        $dtFimAquisicao, $qtmesAfastamento,
        $idControlferias, $qtdiaAfastament,
        $dtInigozoferias,
        $dtfimGozoferias, $dtRetornoferias,
        $qtDiasAbonoFerias, $qtFaltasPeriodo) {
        $msg           = array();
        
        $diasMaxFerias = 30;
        $diasMaxAbono = 10;
        $nr_tipoferias = 1;
        $dsgozoferias = 'planilha importacao';
        $dtmescompetenc = $this->hcmParameters->getDtCompetencia();
        
        $dtIniaquisicao = DateUtil::getDataDeString(substr($dtIniaquisicao,0,10), DateUtil::FORMATO_BRASILEIRO, true);
        $dtFimAquisicao = DateUtil::getDataDeString(substr($dtFimAquisicao,0,10), DateUtil::FORMATO_BRASILEIRO, true);
        $dtInigozoferias = DateUtil::getDataDeString(substr($dtInigozoferias,0,10), DateUtil::FORMATO_BRASILEIRO, true);
        $dtfimGozoferias = DateUtil::getDataDeString(substr($dtfimGozoferias,0,10), DateUtil::FORMATO_BRASILEIRO, true);
        $dtRetornoferias = DateUtil::getDataDeString(substr($dtRetornoferias,0,10), DateUtil::FORMATO_BRASILEIRO, true);
        
        try {
            $objVinculom = $this->entityManager->getRepository(Repositories::GPE_VINCULOM)->findOneBy(
                array('nrorg' => $nrorg, 'nrvinculom' => $nrvinculom));
            if($objVinculom == null) {
                if($cdmatricula){
                    $objVinculom=GpeVinculom::findOneByNrorgAndCdmatricula($nrorg, $cdmatricula);
                    if ($objVinculom == null)
                        $msg[] = array("msg"=>"O código da matrícula $cdmatricula do vínculo não foi encontrado.", "row" => $nrRow);
                    else
                        $nrvinculom = $objVinculom->getNrvinculom();
                }else{
                    if($cdmatriculaesocial){
                        $objVinculoh = $this->entityManager->getRepository(Repositories::GPE_VINCULOH)->retornaVinculoPorMatriculaESocial($nrorg, $nrvinculom, $cdmatriculaesocial, DateUtil::getDataDeString($dtmescompetenc, DateUtil::FORMATO_BRASILEIRO, true), null /*$nrestruturam*/);
                    if ($objVinculoh == null)
                        $msg[] = array("msg"=>"O código da matrícula do eSocial $cdmatriculaesocial não foi encontrado.", "row" => $nrRow);
                        else{
                            $objVinculom = $this->entityManager->getRepository(Repositories::GPE_VINCULOM)->findOneBy(
                                array('nrorg' => $nrorg, 'nrvinculom' => $objVinculoh->getNrvinculom()));
                            $nrvinculom = $objVinculom->getNrvinculom();
                        }
                    }else{
                        $msg[] = array("msg"=>"O código do vínculo, da matrícula do vínculo ou da matrícula do eSocial não foram encontrados. É necessário o preenchimento de pelo menos um desses campos.", "row" => $nrRow);
                    }
                }
            }
            
            //Verificação de registro já existente e Inserção
            if($objVinculom) { //Apenas se houver um vínculo válido
                //Busca ferias para a data de inicio da aquisicao do vínculo para não inserir repetido
                $ferias = $this->entityManager->getRepository(Repositories::FPA_FERIAS)->findOneBy(
                    array('nrorg' => $nrorg, 'nrvinculom' => $nrvinculom, 'dtiniaquisicao' => $dtIniaquisicao));
                
                if($ferias){
                    $gozoFerias = $this->entityManager->getRepository(Repositories::FPA_GOZOFERIAS)->findOneBy(
                    array('nrorg' => $nrorg, 'nrferias' => $ferias->getNrferias(), 'dtinigozoferias' => $dtInigozoferias));
                    
                    if($gozoFerias){
                        $msg[] = array("msg" => "Já existe um gozo de ferias cadastrado para o vínculo $nrvinculom. com a data de inicio $dtInigozoferias Registro não inserido.", "row" => $nrRow);
                    }else{
                        if($dtInigozoferias > $dtfimGozoferias){
                            $msg[] = array("msg"=>"O inicio do gozo de ferias esta maior que a data final para o vinculo $nrvinculom", "row" => $nrRow);
                        }
                        if(!$dtInigozoferias || !$dtfimGozoferias){
                            $msg[] = array("msg"=>"Os campos de inicio e fim de gozo devem ser informados.", "row" => $nrRow);
                        }
                        //Inclui gozo
                        if(count($msg) == 0) {
                            $this->fpaGozoFeriasEntity->incluiGozoFerias($nrorg, $ferias->getNrferias(), $dtInigozoferias, $dtfimGozoferias, $qtDiasAbonoFerias, $dtRetornoferias, $idAdiant13Sal = 'N',$dsgozoferias);
                        }
                    }
                }else{
                    if($dtIniaquisicao > $dtFimAquisicao){
                        $msg[] = array("msg"=>"O inicio das ferias esta maior que a data final para o vinculo $nrvinculom", "row" => $nrRow);
                    }
                    
                    if($idControlferias != 'ABERTO' && $idControlferias != 'ADQUIRIDO' && $idControlferias != 'GOZADO_PARCIAL' && $idControlferias != 'GOZADO_INTEGRAL' && $idControlferias != 'PERDIDO'){
                        $msg[] = array("msg"=>"O status das ferias esta incorreto $idControlferias para o vinculo $nrvinculom", "row" => $nrRow);
                    }
                    if(!$dtIniaquisicao || !$dtFimAquisicao){
                        $msg[] = array("msg"=>"Os campos de inicio e fim de ferias devem ser informados.", "row" => $nrRow);
                    }
                    //Incluir ferias
                    if(count($msg) == 0) {
                        $nrFerias = $this->fpaferiasEntity->incluiFerias($nrorg, $nrvinculom, $dtIniaquisicao, $dtFimAquisicao, $nr_tipoferias, $qtFaltasPeriodo, $diasMaxFerias, $diasMaxAbono, $qtdiaAfastament, $idControlferias);
                    }
                    if($dtInigozoferias > $dtfimGozoferias){
                        $msg[] = array("msg"=>"O inicio do gozo de ferias esta maior que a data final para o vinculo $nrvinculom", "row" => $nrRow);
                    }
                    if(!$dtInigozoferias || !$dtfimGozoferias){
                        $msg[] = array("msg"=>"Os campos de inicio e fim de gozo devem ser informados.", "row" => $nrRow);
                    }
                    //Inclui gozo
                    if(count($msg) == 0) {
                        $this->fpaGozoFeriasEntity->incluiGozoFerias($nrorg, $nrFerias, $dtInigozoferias, $dtfimGozoferias, $qtDiasAbonoFerias, $dtRetornoferias, $idAdiant13Sal = 'N',$dsgozoferias);
                    }
                }
            }
        } catch (\Exception $e) {
            throw $e;
        }
        return $msg;
    }
    
    public function getRescisaoVinculo($nrvinculom) {        
        return $this->entityManager->getRepository(Repositories::GPE_VINCULOM)->findOneBy(array("nrvinculom" => $nrvinculom,"nrorg" => $this->hcmParameters->getNrorgTrabalho()))->getDtrescisaovinc();
    }
}